// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
//
#include <fstream>
#include <util/PlatformUtils.hpp>
#include <util/TransService.hpp>
#include <sax2/SAX2XMLReader.hpp>
#include <sax2/XMLReaderFactory.hpp>
#include <framework/XMLFormatter.hpp>

#include "parrillaJabata.hpp"


// ---------------------------------------------------------------------------
//  Local data
//
//  encodingName
//      The encoding we are to output in. If not set on the command line,
//      then it is defaulted to LATIN1.
//
//  outputFile
//      The path to the file to parser. Set via command line.
//
//  valScheme
//      Indicates what validation scheme to use. It defaults to 'auto', but
//      can be set via the -v= command.
//
//	expandNamespaces
//		Indicates if the output should expand the namespaces Alias with
//		their URI's, defaults to false, can be set via the command line -e
// ---------------------------------------------------------------------------
static const char*              encodingName    = "LATIN1";
static XMLFormatter::UnRepFlags unRepFlags      = XMLFormatter::UnRep_CharRef;
static char*                    outputFile         = NULL;
static SAX2XMLReader::ValSchemes valScheme      = SAX2XMLReader::Val_Auto;
static bool expandNamespaces= false ;
static bool php= false ;
static bool withoutBody = false;
static bool allInOne = false;
static bool forPrinter = false;
static bool hideContinuity = false;
static bool playMp3s = false;
static bool show_links = true;

static ParrillaJabata::Outputs output           = ParrillaJabata::ScheduleHTMLTable;

static char *BODYARGS = "bgcolor=#FFFFFF";
static char *TITLE = NULL;
static char *LINKSCOLOR = NULL;

static const char *TABLEARGS = "align=center border=0 cellspacing=2 cellpadding=0";
static const char *FIRSTCELLARGS = "align=center width=70px height=10px";
static const char *COLHEADARGS = "align=center bgcolor=#301000 width=80px";
static const char *COLHEADFONTARGS = "face=Arial color=#FFFFFF size=1";
static const char *ROWHEADARGS = "align=center bgcolor=#301000 height=30px";
static const char *ROWHEADFONTARGS = "face=Arial color=#FFFFFF size=1";
static const char *STDCELLARGS = "align=center bgcolor=#e4ebff";
static const char *CONTINUITYCELLARGS = "align=center bgcolor=#cdd5ff";
static const char *PROGRAMRECORDFONTARGS = "face=arial color=#FF0000 size=1";
static const char *PROGRAMREPLAYFONTARGS = "face=arial color=#696969 size=1";
static const char *CONTINUITYFONTARGS = "face=arial color=#4F0F1F size=1";

static const char *TABLEARGS4P = "align=center border=1 cellspacing=0 cellpadding=1";
static const char *FIRSTCELLARGS4P = "align=center width=70px height=10px";
static const char *COLHEADARGS4P = "align=center bgcolor=#CCCCCC width=80px";
//static const char *COLHEADFONTARGS4P = "face=Arial color=#000000 size=1";
static const char *COLHEADFONTARGS4P = "class='days4printer'";
static const char *ROWHEADARGS4P = "align=center bgcolor=#CCCCCC height=20px";
//static const char *ROWHEADFONTARGS4P = "face=Arial color=#000000 size=1";
static const char *ROWHEADFONTARGS4P = "class='hours4printer'";
static const char *STDCELLARGS4P = "align=center";
static const char *CONTINUITYCELLARGS4P = "align=center"; //bgcolor=#6F6F6F";
//static const char *PROGRAMRECORDFONTARGS4P = "face=arial color=#000000 size=2";
//static const char *PROGRAMREPLAYFONTARGS4P = "face=arial color=#444444 size=1";
//static const char *CONTINUITYFONTARGS4P = "face=arial color=#FFFFFF size=1";
static const char *PROGRAMRECORDFONTARGS4P = "class='record4printer'";
static const char *PROGRAMREPLAYFONTARGS4P = "class='replay4printer'";
static const char *CONTINUITYFONTARGS4P = "class='conti4printer'";

static const char *MONDAYTEXT = "LUNES";
static const char *TUESDAYTEXT = "MARTES";
static const char *WEDNESDAYTEXT = "MI&Eacute;RCOLES";
static const char *THURSDAYTEXT = "JUEVES";
static const char *FRIDAYTEXT = "VIERNES";
static const char *SATURDAYTEXT = "S&Aacute;BADO";
static const char *SUNDAYTEXT = "DOMINGO";

static const char *toDayString(int i) {
    switch (i) {
        case 1:
            return MONDAYTEXT;
        case 2:
            return TUESDAYTEXT;
        case 3:
            return WEDNESDAYTEXT;
        case 4:
            return THURSDAYTEXT;
        case 5:
            return FRIDAYTEXT;
        case 6:
            return SATURDAYTEXT;
        case 7:
            return SUNDAYTEXT;
        default:
            return NULL;
    }
}

// ---------------------------------------------------------------------------
//  Local helper methods
// ---------------------------------------------------------------------------
static void usage()
{
    cout <<  "\nUso: parrillaJabata [opciones] \n"
             "parrillaJabata genera código HTML y PHP a partir de los ficheros\n"
             "aaaammdd.jSF y aaaammdd.jBF. Genera por defecto documentos completos \n"
             "(es decir, desde <html> hasta </html> ambas etiquetas incluidas) pero \n"
             "la opción -withoutbody elimina todo lo prescindible,  y sus horarios de emision.\n\n"
	     "Commands:\n"
	     "    -links      Prints an example configuration file for links\n"
	     "    -strings    Prints string configuration file to the stdout\n"
	     "    -? --help   Show this helpy stuff\n"
             "Options:\n"
	     "    -header=xx  Attach the selected file before the schedule table.\n"
	     "    -footer=xx  Append the desired file at the end of the table.\n"
	     "    -links=xx   Read links URLs from 'xx' file.\n"
	     "    -strings=xx Read strings from 'xx' file.\n"
             "    -o=xxx      [schedule*|sch4printer|programs|programslinks|rawids|PROGRAMID]\n"
	     "    -nolinks    Hide links\n"
             "    -beginat=xx Hora de comienzo para la parrilla generada.\n"
             "    -withoutbody\n"
             "    -allinone   Los hiperenlaces de la parrilla apuntan al mismo\n"
	     "                fichero, en el que se encuentran todos los programas.\n" 
             "    -title=\"...\"\n"
             "    -bodyargs=\"...\"\n"
             "    -playmp3s\n"
             "		-linkscolor=\"#rrggbb\" (sólo para -o=programslinks) \n"
             "    -hidecontinuity\n"
             "    -php\n"
             "    -u=xxx      Handle unrepresentable chars [fail | rep | ref*]\n"
             "    -v=xxx      Validation scheme [always | never | auto*]\n"
             "    -e          Expand Namespace Alias with URI's.\n"
             "    -x=XXX      Use a particular encoding for output (LATIN1*).\n"
             "  * = Default if not provided explicitly\n\n"
             "The parser has intrinsic support for the following encodings:\n"
             "    UTF-8, USASCII, ISO8859-1, UTF-16[BL]E, UCS-4[BL]E,\n"
             "    WINDOWS-1252, IBM1140, IBM037\n"
         <<  endl;
}

/* XXX this must be fixed, needs strings must be accessed using an array. */
bool setString(char *name,char *rest)
{
	if (!strcmp(name,"BODYARGS"))
		BODYARGS=strdup(rest); else
	if (!strcmp(name,"TITLE"))
		TITLE=strdup(rest); else
	if (!strcmp(name,"LINKSCOLOR"))
		LINKSCOLOR=strdup(rest); else
	if (!strcmp(name,"TABLEARGS"))
		TABLEARGS=strdup(rest); else
	if (!strcmp(name,"FIRSTCELLARGS"))
		FIRSTCELLARGS=strdup(rest); else
	if (!strcmp(name,"COLHEADARGS"))
		COLHEADARGS=strdup(rest); else
	if (!strcmp(name,"COLHEADFONTARGS"))
		COLHEADFONTARGS=strdup(rest); else
	if (!strcmp(name,"ROWHEADARGS"))
		ROWHEADARGS=strdup(rest); else
	if (!strcmp(name,"ROWHEADFONTARGS"))
		ROWHEADFONTARGS=strdup(rest); else
	if (!strcmp(name,"STDCELLARGS"))
		STDCELLARGS=strdup(rest); else
	if (!strcmp(name,"CONTINUITYCELLARGS"))
		CONTINUITYCELLARGS=strdup(rest); else
	if (!strcmp(name,"PROGRAMRECORDFONTARGS"))
		PROGRAMRECORDFONTARGS=strdup(rest); else
	if (!strcmp(name,"PROGRAMREPLAYFONTARGS"))
		PROGRAMREPLAYFONTARGS=strdup(rest); else
	if (!strcmp(name,"CONTINUITYFONTARGS"))
		CONTINUITYFONTARGS=strdup(rest); else
	if (!strcmp(name,"TABLEARGS4P"))
		TABLEARGS4P=strdup(rest); else
	if (!strcmp(name,"FIRSTCELLARGS4P"))
		FIRSTCELLARGS4P=strdup(rest); else
	if (!strcmp(name,"COLHEADARGS4P"))
		COLHEADARGS4P=strdup(rest); else
	if (!strcmp(name,"COLHEADFONTARGS4P"))
		COLHEADFONTARGS4P=strdup(rest); else
	if (!strcmp(name,"ROWHEADARGS4P"))
		ROWHEADARGS4P=strdup(rest); else
	if (!strcmp(name,"ROWHEADFONTARGS4P"))
		ROWHEADFONTARGS4P=strdup(rest); else
	if (!strcmp(name,"STDCELLARGS4P"))
		STDCELLARGS4P=strdup(rest); else
	if (!strcmp(name,"CONTINUITYCELLARGS4P"))
		CONTINUITYCELLARGS4P=strdup(rest); else
	if (!strcmp(name,"PROGRAMRECORDFONTARGS4P"))
		PROGRAMRECORDFONTARGS4P=strdup(rest); else
	if (!strcmp(name,"PROGRAMREPLAYFONTARGS4P"))
		PROGRAMREPLAYFONTARGS4P=strdup(rest); else
	if (!strcmp(name,"CONTINUITYFONTARGS4P"))
		CONTINUITYFONTARGS4P=strdup(rest); else
	if (!strcmp(name,"MONDAYTEXT"))
		MONDAYTEXT=strdup(rest); else
	if (!strcmp(name,"TUESDAYTEXT"))
		TUESDAYTEXT=strdup(rest); else
	if (!strcmp(name,"WEDNESDAYTEXT"))
		WEDNESDAYTEXT=strdup(rest); else
	if (!strcmp(name,"THURSDAYTEXT"))
		THURSDAYTEXT=strdup(rest); else
	if (!strcmp(name,"FRIDAYTEXT"))
		FRIDAYTEXT=strdup(rest); else
	if (!strcmp(name,"SATURDAYTEXT"))
		SATURDAYTEXT=strdup(rest); else
	if (!strcmp(name,"SUNDAYTEXT"))
		SUNDAYTEXT=strdup(rest); else
	return false;

	return true;
}

void showStrings()
{
	cout <<
	"BODYARGS "      << BODYARGS << endl <<
	"TITLE "         << TITLE << endl <<
	"LINKSCOLOR "    << LINKSCOLOR << endl <<
	"TABLEARGS "     << TABLEARGS << endl <<
	"FIRSTCELLARGS " << FIRSTCELLARGS << endl <<
	"COLHEADARGS "   << COLHEADARGS << endl <<
	"COLHEADFONTARGS " << COLHEADFONTARGS << endl <<
	"ROWHEADARGS " << ROWHEADARGS << endl <<
	"ROWHEADFONTARGS " << ROWHEADFONTARGS << endl <<
	"STDCELLARGS " << STDCELLARGS << endl <<
	"CONTINUITYCELLARGS " << CONTINUITYCELLARGS << endl <<
	"PROGRAMRECORDFONTARGS " << PROGRAMRECORDFONTARGS << endl <<
	"PROGRAMREPLAYFONTARGS " << PROGRAMREPLAYFONTARGS << endl <<
	"CONTINUITYFONTARGS " <<CONTINUITYFONTARGS << endl <<
	"TABLEARGS4P " <<TABLEARGS4P << endl <<
	"FIRSTCELLARGS4P " <<FIRSTCELLARGS4P << endl <<
	"COLHEADARGS4P " <<COLHEADARGS4P << endl <<
	"COLHEADFONTARGS4P " <<COLHEADFONTARGS4P << endl <<
	"ROWHEADARGS4P " << ROWHEADARGS4P << endl <<
	"ROWHEADFONTARGS4P " << ROWHEADFONTARGS4P << endl <<
	"STDCELLARGS4P " <<STDCELLARGS4P << endl <<
	"CONTINUITYCELLARGS4P " <<CONTINUITYCELLARGS4P << endl <<
	"PROGRAMRECORDFONTARGS4P " << PROGRAMRECORDFONTARGS4P << endl <<
	"PROGRAMREPLAYFONTARGS4P " << PROGRAMREPLAYFONTARGS4P << endl <<
	"CONTINUITYFONTARGS4P " <<CONTINUITYFONTARGS4P << endl <<
	"MONDAYTEXT " << MONDAYTEXT << endl <<
	"TUESDAYTEXT " << TUESDAYTEXT << endl <<
	"WEDNESDAYTEXT " <<WEDNESDAYTEXT << endl <<
	"THURSDAYTEXT " << THURSDAYTEXT << endl <<
	"FRIDAYTEXT " << FRIDAYTEXT << endl <<
	"SATURDAYTEXT " << SATURDAYTEXT << endl <<
	"SUNDAYTEXT " <<SUNDAYTEXT << endl 
	;
}

void loadStrings(char *file)
{
	char buffer[1024];
	char name[1024];
	char *rest;
	bool ok=false;

	std::ifstream is(file);
	if (is.is_open())
	{
	  while(!is.eof())
	  {
		is.getline(buffer,1000);
		if (is.eof()) continue;
		/* get name */
		sscanf(buffer,"%s",name);
		/* get string */
		rest=strstr(buffer,name);
		while(rest[0]!='\0' && rest[0]!=' ') *rest++;
		while(rest[0]!='\0' && rest[0]==' ') *rest++;

		ok = setString(name,rest);
		if (!ok) break;
	  }
	  is.close();

	  if (!ok)
	  {
		cerr << "Invalid keyword in strings file" << endl;
		exit(1);
	  }
	} else {
		cerr << "Cannot open strings file '" << file << "'."<< endl;
		exit(1);
	}
}


char *links_file="";
char g_buf[1024];    /* global variable used by getLinkFor as return value,
                        weird but quick fix to what was there before,
                        which was returning a local variable =:-} */

char *getLinkFor(int n)
{
	char buffer[1024];
	char name[1024];
	char *rest;

	g_buf[0]='\0';

	if (links_file[0]=='\0')
		sprintf(g_buf,"%d.html",n);
	else {

	std::ifstream is(links_file);
	if (is.is_open())
	{
	  int mynumber;

	  while(!is.eof())
	  {
		is.getline(buffer,1000);
		if (is.eof()) continue;
		/* get name */
		sscanf(buffer,"%s",name);

		/* get string */
		rest=strstr(buffer,name);
		while(rest[0]!='\0' && rest[0]!=' ') *rest++;
		while(rest[0]!='\0' && rest[0]==' ') *rest++;

		if (!strcmp(name,"default"))
		{
		   strcpy(g_buf,rest);
		} else {
		  mynumber=atoi(name);
		  if (mynumber == n)
		  {
		  if (rest[0]=='\0')
		    {
			cerr << "no link found for code: " << n << "." << endl;
		    } else {
			strcpy(g_buf,rest);
		    }
		  }
		break;
		}
	    }
	  is.close();
	} else {
		cerr << "Cannot open links file '" << links_file << "'."<< endl;
		exit(1);
	}
	}

	return g_buf;
}

void showLinks()
{
	cout
	<< "default /index.html" << endl
	<< "1 http://www.google.com/" << endl
	<< "2 http://www.sindominio.net/" << endl;
}

char *footer_file=0;
char *header_file=0;

void attachFile(char *file)
{
	char buffer[1024];

	if (file==NULL) return;

	std::ifstream is(file);

	if (is.is_open())
	{
	  while(!is.eof())
	  {
		is.getline(buffer,1000);
		cout << buffer << endl;
	  }
	  is.close();
	} else {
		cerr << "Cannot find attached file '"<<file <<"'." << endl;
		exit(1);
	}
}

// ---------------------------------------------------------------------------
//  Program entry point
// ---------------------------------------------------------------------------
int main(int argC, char* argV[])
{
    char *strings_file="";

    // Initialize the XML4C2 system
    try
    {
         XMLPlatformUtils::Initialize();
    }

    catch (const XMLException& toCatch)
    {
         cerr << "Error during initialization! :\n"
              << StrX(toCatch.getMessage()) << endl;
         return 1;
    }
    switch (argC) {
        case 2:
            if (!strcmp(argV[1],"-links"))
            {
                showLinks();
                exit(0);
            }

            if (!strcmp(argV[1],"-strings"))
            {
                showStrings();
                exit(0);
            }
#define helpinst \
    {\
        usage();\
        XMLPlatformUtils::Terminate();\
        return 2;\
    }
// Watch for special case help request
            if(!strcmp(argV[1],"--help") || !strcmp(argV[1],"-h") ) helpinst;
            break;
        case 3:
            if(!strcmp(argV[1], "-?")) helpinst;
    }
#undef helpinst


    int idBlock=-1;
    int beginAt=0;

    int parmInd;

    for (parmInd = 1; parmInd < argC; parmInd++)
    {
        // Break out on first parm not starting with a dash
        if (argV[parmInd][0] != '-')
            break;

        if (!strncmp(argV[parmInd], "-o=", 3) ||  
	    !strncmp(argV[parmInd], "-O=", 3))
        {
            const char* const parm = &argV[parmInd][3];

            if (!strcmp(parm, "programs"))
                output = ParrillaJabata::ProgramsHTMLTable;
            else if (!strcmp(parm, "schedule"))
                output = ParrillaJabata::ScheduleHTMLTable;
            else if (!strcmp(parm, "rawids"))
                output = ParrillaJabata::ProgramRawIDs;
            else if (!strcmp(parm, "sch4printer")) {
                forPrinter=true;
                output = ParrillaJabata::ScheduleHTMLTable;
            }
            else if (!strcmp(parm, "programslinks"))
                output = ParrillaJabata::ProgramsHTMLLinks;
            else {
                idBlock = atoi(&argV[parmInd][3]);
                if(idBlock>0) {
                  output = ParrillaJabata::ProgramHTMLTable;
                }
                else {
                  cerr << "Unknown -o= value: " << parm << endl;
                  return 2;
                }
            }
        }
	else if (!strncmp(argV[parmInd], "-links=",7))
	{
		links_file=&argV[parmInd][7];
	}
	else if (!strncmp(argV[parmInd], "-nolinks",8))
	{
		show_links=false;
	}
	else if (!strncmp(argV[parmInd], "-strings=",9))
	{
		strings_file=&argV[parmInd][9];
	}
	else if (!strncmp(argV[parmInd], "-header=",8))
	{
		header_file=&argV[parmInd][8];
	}
	else if (!strncmp(argV[parmInd], "-footer=",8))
	{
		footer_file=&argV[parmInd][8];
	}
        else if (!strncmp(argV[parmInd], "-bodyargs=", 10)
        ||  !strncmp(argV[parmInd], "-BODYARGS=", 10))
        {
            BODYARGS = &argV[parmInd][10];
        }
        else if (!strncmp(argV[parmInd], "-title=", 7)
        ||  !strncmp(argV[parmInd], "-TITLE=", 7))
        {
            TITLE = &argV[parmInd][7];
        }
        else if (!strncmp(argV[parmInd], "-linkscolor=", 12)
        ||  !strncmp(argV[parmInd], "-LINKSCOLOR=", 12))
        {
            LINKSCOLOR = &argV[parmInd][12];
        }
        else if (!strncmp(argV[parmInd], "-php", 4)
        ||  !strncmp(argV[parmInd], "-PHP", 4))
        {
            php = true;
        }
        else if (!strncmp(argV[parmInd], "-playmp3s", 9)
        ||  !strncmp(argV[parmInd], "-PLAYMP3S", 9))
        {
            playMp3s = true;
        }
        else if (!strncmp(argV[parmInd], "-allinone", 9)
        ||  !strncmp(argV[parmInd], "-ALLINONE", 9))
        {
            allInOne = true;
        }
        else if (!strncmp(argV[parmInd], "-hidecontinuity", 15)
        ||  !strncmp(argV[parmInd], "-HIDECONTINUITY", 15))
        {
            hideContinuity = true;
        }
        else if (!strncmp(argV[parmInd], "-withoutbody", 12)
        ||  !strncmp(argV[parmInd], "-WITHOUTBODY", 12))
        {
            withoutBody = true;
        }
        else if (!strncmp(argV[parmInd], "-beginat=", 9)
        ||  !strncmp(argV[parmInd], "-BEGINAT=", 9))
        {
          beginAt = atoi(&argV[parmInd][9]);
          if((beginAt<0) || (beginAt>24)) {
             cerr << "Wrong -beginat= value: " << argV[parmInd][9] << endl;
             return 2;
          }
	  
        }
        else if (!strncmp(argV[parmInd], "-v=", 3)
        ||  !strncmp(argV[parmInd], "-V=", 3))
        {
            const char* const parm = &argV[parmInd][3];

            if (!strcmp(parm, "never"))
                valScheme = SAX2XMLReader::Val_Never;
            else if (!strcmp(parm, "auto"))
                valScheme = SAX2XMLReader::Val_Auto;
            else if (!strcmp(parm, "always"))
                valScheme = SAX2XMLReader::Val_Always;
            else
            {
                cerr << "Unknown -v= value: " << parm << endl;
                return 2;
            }
        }
         else if (!strcmp(argV[parmInd], "-e")
              ||  !strcmp(argV[parmInd], "-E"))
        {
            expandNamespaces = true;
        }
         else if (!strncmp(argV[parmInd], "-x=", 3)
              ||  !strncmp(argV[parmInd], "-X=", 3))
        {
            // Get out the encoding name
            encodingName = &argV[parmInd][3];
        }
         else if (!strncmp(argV[parmInd], "-u=", 3)
              ||  !strncmp(argV[parmInd], "-U=", 3))
        {
            const char* const parm = &argV[parmInd][3];

            if (!strcmp(parm, "fail"))
                unRepFlags = XMLFormatter::UnRep_Fail;
            else if (!strcmp(parm, "rep"))
                unRepFlags = XMLFormatter::UnRep_Replace;
            else if (!strcmp(parm, "ref"))
                unRepFlags = XMLFormatter::UnRep_CharRef;
            else
            {
                cerr << "Unknown -u= value: " << parm << endl;
                return 2;
            }
        }
         else
        {
            cerr << "Unknown option '" << argV[parmInd]
                 << "', ignoring it\n" << endl;
            return 2;
        }
    }

    //
    //  And now we could have only one parameter left and it should be
    //  the output file.
    //
    if ((parmInd != argC)&&(parmInd + 1 != argC))
    {
        usage();
        return 1;
    }
    else if (parmInd + 1 == argC)
      outputFile = argV[parmInd];

    /* load strings */
    if (strings_file[0])
    loadStrings(strings_file);

    /* do work */
    SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();
	  if (valScheme == SAX2XMLReader::Val_Auto)
	  {
	    parser->setFeature(XMLString::transcode("http://xml.org/sax/features/validation"), true);
	    parser->setFeature(XMLString::transcode("http://apache.org/xml/features/validation/dynamic"), true);
	  }
      if (valScheme == SAX2XMLReader::Val_Never)
	  {
	    parser->setFeature(XMLString::transcode("http://xml.org/sax/features/validation"), false);
	  }
	
	  if (valScheme == SAX2XMLReader::Val_Always)
	  {
	    parser->setFeature(XMLString::transcode("http://xml.org/sax/features/validation"), true);
	    parser->setFeature(XMLString::transcode("http://apache.org/xml/features/validation/dynamic"), false);
	  }

	  JabatoSchedule sch;
    if(!sch.initSchedule(parser,false)) {
      cerr << "\nError during schedule initialization!\n";
      return -1;
	  }
	

	  ParrillaJabata parrilla(&sch);

    parrilla.blockId = idBlock;
	  parrilla.bodyArgs = BODYARGS;
	  parrilla.withPHPCode = php;
	  parrilla.withoutBody = withoutBody;
	  parrilla.allInOne = allInOne;
	  parrilla.beginAt = beginAt;
	  parrilla.forPrinter = forPrinter;
	  parrilla.hideContinuity = hideContinuity;
	  parrilla.title = TITLE;
	  parrilla.linkscolor = LINKSCOLOR;
	  parrilla.playMp3s = playMp3s;
	  switch(output) {
	    case ParrillaJabata::ProgramHTMLTable:
	      parrilla.programHTMLTable(outputFile);
	      break;
	    case ParrillaJabata::ProgramsHTMLTable:
	      parrilla.programsHTMLTable(outputFile);
	      break;
	    case ParrillaJabata::ProgramsHTMLLinks:
	      parrilla.programsHTMLLinks(outputFile);
	      break;
	    case ParrillaJabata::ScheduleHTMLTable:
	      parrilla.scheduleHTMLTable(outputFile);
	      break;
	    case ParrillaJabata::ProgramRawIDs:
	      parrilla.programRawIDs(outputFile);
	      break;
	  }
	

    // And call the termination method
    XMLPlatformUtils::Terminate();

    return 0;

}


ParrillaJabata::ParrillaJabata(JabatoSchedule *schedule) {
  sch = schedule;
  bodyArgs = NULL;
  title = NULL;
  linkscolor = NULL;
  withoutBody=false;
  forPrinter=false;
  allInOne=false;
  withPHPCode=false;
  hideContinuity=false;
  blockId=-1;
}


void
ParrillaJabata::scheduleHTMLTable(char *file) {
  if(!withoutBody) {
  	cout << "<html><head>" << 
	  "<title>" << endl <<
          ((title?title:
	          sch->getStationName()?sch->getStationName():
                        "KJabata")) << endl <<
	  "</title>" << 
	  "<meta name=\"Author\" content=\"KJabata\">" << 
	  "<link rel=\"stylesheet\" href=\"wjabata.css\" type=\"text/css\">"<< 
	  "</head>" << endl;
  	cout << "<body " << bodyArgs  << ">" << endl;
  }

  attachFile(header_file);

  if(forPrinter) {
    cout << "<center><span class=\"title4printer\">" << title << 
	    "</span></center><br>";
    cout << "<table " << TABLEARGS4P << ">" << endl;
    cout << "<tr>" << endl
       << " <td " << FIRSTCELLARGS4P << ">&nbsp;</td>" << endl
       << " <td " << COLHEADARGS4P << "><font " << COLHEADFONTARGS4P << "><b>" << MONDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS4P << "><font " << COLHEADFONTARGS4P << "><b>" << TUESDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS4P << "><font " << COLHEADFONTARGS4P << "><b>" << WEDNESDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS4P << "><font " << COLHEADFONTARGS4P << "><b>" << THURSDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS4P << "><font " << COLHEADFONTARGS4P << "><b>" << FRIDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS4P << "><font " << COLHEADFONTARGS4P << "><b>" << SATURDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS4P << "><font " << COLHEADFONTARGS4P << "><b>" << SUNDAYTEXT << "</b></font></td>" << endl;
  }
  else {
    if(withPHPCode)
      cout << "<?  if(file_exists(\"scheduletop.php\")) require \"scheduletop.php\" ?>" << endl;
	  cout << "<table " << TABLEARGS << ">" << endl;
	  cout << "<tr>" << endl
       << " <td " << FIRSTCELLARGS << ">&nbsp;</td>" << endl
       << " <td " << COLHEADARGS << "><font " << COLHEADFONTARGS << "><b>" << MONDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS << "><font " << COLHEADFONTARGS << "><b>" << TUESDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS << "><font " << COLHEADFONTARGS << "><b>" << WEDNESDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS << "><font " << COLHEADFONTARGS << "><b>" << THURSDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS << "><font " << COLHEADFONTARGS << "><b>" << FRIDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS << "><font " << COLHEADFONTARGS << "><b>" << SATURDAYTEXT << "</b></font></td>" << endl
       << " <td " << COLHEADARGS << "><font " << COLHEADFONTARGS << "><b>" << SUNDAYTEXT << "</b></font></td>" << endl;
  }
  int i;
  int lastHourIds[7] = {-2,-2,-2,-2,-2,-2,-2};
  int rows,x,xhh,xmm;
  const char *fontargs;
  const char *cellargs;


  int hora;
  int hh,mm;
  int dia;
  int diaaux;
  int blockId;
  beginAt=beginAt*2;

  for(int h=0; h<48; h++) {
//    if(forPrinter && (h + beginAt)>23)
//      break;
//    else
    hora = (h + beginAt) % 48;
    hh=hora/2;
    if(hora%2)
      mm=41;
    else
      mm=11;
    cout << "<tr>" << endl;

    if(mm==11) {
      if(forPrinter) {
        cout << " <td rowspan=2 " << ROWHEADARGS4P << "><font " 
	     << ROWHEADFONTARGS4P << "><b>"
             << hh << ":00-" << hh+1 << ":00</b></font></td>" << endl;
      }
      else {
        cout << " <td rowspan=2 " << ROWHEADARGS << "><font " 
	     << ROWHEADFONTARGS << "><b>"
             << hh << ":00-" << hh+1 << ":00</b></font></td>" << endl;
      }
    }

    for(int d=0; d<7; d++) {
      dia=d+1;
      if(hora<beginAt) {
        dia++;
	if(dia==8) dia=1;
      }
      
      i = sch->getBlockPosition(dia, hh, mm);
      if(forPrinter) {
        if(i==0) {
  	  fontargs = CONTINUITYFONTARGS4P;
    	  cellargs = CONTINUITYCELLARGS4P;
      	}
      	else if(sch->getCurrentStrip()->recordingStrip) {
          fontargs = PROGRAMRECORDFONTARGS4P;
          cellargs = STDCELLARGS4P;
      	} else {
          fontargs = PROGRAMREPLAYFONTARGS4P;
          cellargs = STDCELLARGS4P;
      	}
      }
      else {
	if(i==0) {
  	  fontargs = CONTINUITYFONTARGS;
    	  cellargs = CONTINUITYCELLARGS;
      	}
      	else if(sch->getCurrentStrip()->recordingStrip) {
          fontargs = PROGRAMRECORDFONTARGS;
          cellargs = STDCELLARGS;
      	} else {
          fontargs = PROGRAMREPLAYFONTARGS;
          cellargs = STDCELLARGS;
      	}
      }

      if(lastHourIds[dia-1]!=i) {
        x = hora;
        if(((x+1)%48)<beginAt) {
          diaaux=dia+1;
	  if(diaaux==8) diaaux=1;
	}
	else
	  diaaux=dia;
	
	xhh=(x+1)/2;
	xmm=((x+1)%2?41:11);
	
        while((x<(48+beginAt)) && 
	      (i == sch->getBlockPosition(diaaux, (xhh%24), xmm))) {
	  x++;
          if(((x+1)%48)<beginAt) {
            diaaux=dia+1;
	    if(diaaux==8) diaaux=1;
	  }
	  xhh=(x+1)/2;
  	  xmm=((x+1)%2?41:11);
	}
        rows = x - hora;

        cout << " <td " << cellargs;

        if(rows) cout << " rowspan=" << rows + 1;

        blockId = sch->getBlockId(i);
//	blockId = 0;
        if(( blockId > 0) && (blockId < 1000)) {

	  cout << ">";

          if(allInOne) {
            if(withPHPCode)
	    {
	      if (show_links)
	      	cout << "<a href=\"programas.php#" << i << "\">";
	      cout << "<font " << fontargs << "><b>"
    	           << (i>=0?sch->getBlockName(i):"?") 
		   << "</b></font></a></td>" << endl;
	    }
    	    else
	    {
		if (show_links)
              	cout << "<a href=\"programas.html#" << i << "\">";
	        cout << "<font " << fontargs << "><b>"
                     << (i>=0?sch->getBlockName(i):"?") 
		     << "</b></font></a></td>" << endl;
	    }
          }
          else {
            if(withPHPCode)
	    {
	      if (show_links)
	      cout << "<a href=\"/varjabata/" << blockId << ".php\">";
	      cout << "<font " << fontargs << "><b>"
    	           << (i>=0?sch->getBlockName(i):"?") 
		   << "</b></font></a></td>" << endl;
	     }
    	    else
	     {
	      if (show_links)
	      cout << "<a href=\"" << getLinkFor(blockId) << "\">";
	      cout << "<font " << fontargs << "><b>"
    	           << (i>=0?sch->getBlockName(i):"?") 
		   << "</b></font></a></td>" << endl;
	     }
          }
        }
        else {
          if(hideContinuity) 
            cout << ">&nbsp;</td>" << endl;
  	  else
	    cout << "><font " << fontargs << ">"
  	         << (i>=0?sch->getBlockName(i):"?") 
	         << "</font> </td>" << endl;
        }
        lastHourIds[dia-1] = i;
      }
    }
  }

  cout << "</table>" << endl;

  attachFile(footer_file);

  if(withPHPCode && (!forPrinter))
    cout << "<?  if(file_exists(\"schedulebottom.php\")) require \"schedulebottom.php\" ?>" << endl;

  if(!withoutBody) {
    cout << "</body>" << endl;
    cout << "</html>" << endl;
  }
}

void
ParrillaJabata::programsHTMLTable(char *file) {
  char 			  emisiones[512];
  char 			  reemisiones[512];
  char        *str;
  int         indE,indR,ind;
  int				diff;
  JabatoStrip *strip;
  JabatoBlock *blk;
  JabatoSound sound;

  if(withPHPCode) {
    cout << "<?" << endl;
    cout << "function echoDownloads($filter) {" << endl;
    cout << "  if(file_exists(\"download\"))" << endl;
    cout << "    $direct = opendir(\"download\");" << endl;
    cout << "  else" << endl;
    cout << "    $direct = opendir(\".\");" << endl;
    cout << "  $fic = readdir($direct);" << endl;
  	cout << "  $cabecera = 1;" << endl;
    cout << "  while($fic) {" << endl;
  	cout << "    $fic = readdir($direct);" << endl;
  	cout << "    if(ereg(\"$filter$\",$fic)) {" << endl;
  	cout << "      if($cabecera) {" << endl;
    cout << "        echo \"<b>Posibles descargas</b>:<br>\";" << endl;
    cout << "        $cabecera = 0; }" << endl;
    cout << "      if(file_exists(\"img/sound.png\"))" << endl;
    cout << "        echo \"<img src=\\\"img/sound.png\\\" hspace=5 align=\\\"bottom\\\" >\";" << endl;
    cout << "      echo \"<a href=\\\"download/\".urlencode($fic).\"\\\"><b>$fic</b> - \";" << endl;
    cout << "      echo number_format(filesize(\"download/\".$fic),0,\"\",\".\").\" Bytes \";" << endl;
    cout << "      echo exec(\"../../mp3info \\\"download/$fic\\\"\").\"</a><br>\";" << endl;
    cout << "      if(file_exists(\"download/\".$fic.\".desc\"))" << endl;
    cout << "        include(\"download/\".$fic.\".desc\");" << endl;
    cout << "    }" << endl;
    cout << "  }" << endl;
    cout << "}" << endl;
    cout << "?>" << endl;
  }

  if(!withoutBody) {
    cout << "<html><head><title>" << endl <<
         ((title?title:
	     sch->getStationName()?sch->getStationName():
	                          "KJabata")) << endl << 
	 "</title>" <<
	 "<meta name=\"Author\" content=\"Kontinuidad Jabata\">" <<
	 "<link rel=\"stylesheet\" href=\"wjabata.css\" type=\"text/css\">" << 
	 "</head>" << endl << 
	 "<body " << bodyArgs  << ">" << endl;
  }

  cout << "<a name=\"0\">&nbsp;" << endl ;

  if(withPHPCode)
    cout << "<?  if(file_exists(\"programstop.php\")) require \"programstop.php\" ?>" << endl;

  for(uint i=1; i<sch->blocksCount(); i++) {
    blk = sch->getBlockAt(i);

    if(blk->getId()<1000) {
      emisiones[0]=reemisiones[0]=0;
      indE = indR = 0;
      for(int d=1; d<8; d++) {
        for(int h=0; h<24; h++) {
          if((int)i==sch->getBlockPosition(d, h, 0)){
            strip = sch->getCurrentStrip();
            if((!strip->recordingStrip) && (blk->getProgramFile())) {
              str = (char *)&reemisiones;
              ind = indR;
            }
            else {
              str = (char *)&emisiones;
              ind = indE;
            }
						
	    if(str[0]) {
              sprintf(str,"%s <br></b> y los <b>%s DE %d:%02d A %d:%02d", str, toDayString(d),
                         strip->beginAtHour,strip->beginAtMinute,
                         strip->endAtHour,strip->endAtMinute);
						}
						else {
              sprintf(str,"%s DE %d:%02d A %d:%02d", toDayString(d),
                         strip->beginAtHour,strip->beginAtMinute,
                         strip->endAtHour,strip->endAtMinute);
            }
          }
          diff = sch->getCurrentStrip()->endAtHour - sch->getCurrentStrip()->beginAtHour;
          if(diff>1) h+=(diff - 1);
        }
      }

      cout << "<a name=\"" << i << "\">&nbsp;" << endl ;

      cout << "<table border=1 cellspacing=4 cellpadding=8>" << endl;
      cout << "<tr>" << endl;
      if(blk->getBackImage()!=NULL)
        cout << " <td background=\"" << blk->getBackImage() << "\">" << endl;
      else
        cout << " <td background=\"img/fclaro.png\">" << endl;

      cout << " <font color=#551500 size=2><b>";
      if(emisiones[0])
        cout << emisiones;
      else {
        if(reemisiones[0]) {
          cout << reemisiones;
	  reemisiones[0]=0;
	}
	else
          cout << "Hist&oacute;rico" << endl;
      }
      cout << "</b></font><HR>" << endl;

      cout << " <center><font color=#551500 size=5>" << blk->getName() << "</font></center><HR>" << endl;
      if(blk->getImage()!=NULL)
        cout << " <img src=\"" << blk->getImage() << "\" hspace=10 align=\"left\" alt=\"Imagen " << blk->getName() << "\">" << endl;

      cout << " <p><font color=#000000><b>" << blk->getDescription() << "</b></font></p>" << endl;
      cout << " <br clear=\"left\"><br>";

      if(blk->getJabatos()!=NULL)
        cout << "<p>Culpable/s: <b>" << blk->getJabatos() << "</b></p>" << endl;

      if(blk->getEmail()!=NULL)
        cout << "<p>Correo electr&oacute;nico: <a href=\"mailto:" << blk->getEmail() << "\">"
             << blk->getEmail() << "</a></p>" << endl;

      if(blk->getHRef()!=NULL)
        cout << "<p>P&aacute;gina G&uuml;eb: <a href=\"" << blk->getHRef() << "\">"
             << blk->getHRef() << "</a></p>" << endl;

      if((blk->getProgramFile()) ||(reemisiones[0])) {
        cout << "<HR><font color=#551500 size=2><div align=\"right\"><b>" << blk->getName() << "</b>";
        if(reemisiones[0])
          cout << " se repite los <b>" << reemisiones;
        else
          cout << " sonar&aacute; como parte de <b>"
               << sch->getDefaultBlock()->getName();
      }
      cout << "</b></div></font>" << endl;

      if((withPHPCode) && (blk->getProgramFile())) {
        cout << "<hr><font color=#000055 size=2>" << endl;
        sound.setFullPath(blk->getProgramFile());
        cout << "<? echoDownloads(\"" << sound.getName() << ".mp3\"); ?>" << endl;
        cout << "</font>" << endl;

      }

      cout << "</td></table>" << endl;

			cout << "<a href=\"#0\"><img src=\"goup.png\" border=0 vspace=6></a>" << endl;
    }
  }

  if(withPHPCode)
    cout << "<?  if(file_exists(\"programsbottom.php\")) require \"programsbottom.php\" ?>" << endl;

  if(!withoutBody) {
    cout << "</body>" << endl;
    cout << "</html>" << endl;
  }
}

void
ParrillaJabata::programHTMLTable(char *file) {
  char 			  emisiones[512];
  char 			  reemisiones[512];
  char        *str;
  int         indE,indR,ind;
  int					diff;
  JabatoStrip *strip;
  JabatoBlock *blk;
  JabatoSound sound;

  if(withPHPCode) {
    cout << "<?" << endl;
    cout << "function echoDownloads($filter) {" << endl;
    cout << "  if(file_exists(\"download\"))" << endl;
    cout << "    $direct = opendir(\"download\");" << endl;
    cout << "  else" << endl;
    cout << "    $direct = opendir(\".\");" << endl;
    cout << "  $fic = readdir($direct);" << endl;
  	cout << "  $cabecera = 1;" << endl;
    cout << "  while($fic) {" << endl;
  	cout << "    $fic = readdir($direct);" << endl;
  	cout << "    if(ereg(\"$filter$\",$fic)) {" << endl;
  	cout << "      if($cabecera) {" << endl;
    cout << "        echo \"<b>Posibles descargas</b>:<br>\";" << endl;
    cout << "        $cabecera = 0; }" << endl;
    cout << "      if(file_exists(\"img/sound.png\"))" << endl;
    cout << "        echo \"<img src=\\\"img/sound.png\\\" hspace=5 align=\\\"bottom\\\" >\";" << endl;
    if(playMp3s && (blockId!=-1))
      cout << "      echo \"<a href=\\\" << blockId << \".php?playmp3=download/\".urlencode($fic).\"\\\"><b>$fic</b> - \";" << endl;
    else
      cout << "      echo \"<a href=\\\"download/\".urlencode($fic).\"\\\"><b>$fic</b> - \";" << endl;
    cout << "      echo number_format(filesize(\"download/\".$fic),0,\"\",\".\").\" Bytes \";" << endl;
    cout << "      echo exec(\"../../mp3info -f \\\"(%mm %ss) %b kbit/s %M\\\" \\\"download/$fic\\\"\").\"</a><br>\";" << endl;
    cout << "      if(file_exists(\"download/\".$fic.\".desc\"))" << endl;
    cout << "        include(\"download/\".$fic.\".desc\");" << endl;
    cout << "    }" << endl;
    cout << "  }" << endl;
    cout << "}" << endl;
    cout << "?>" << endl;
  }

  for(uint i=1; i<sch->blocksCount(); i++) {
    blk = sch->getBlockAt(i);

    if(blk->getId()==blockId) {
      if(!withoutBody) {
        cout << "<html><head>" << 
	 "<title>" << endl <<
            ((title?title:blk->getName())) << ": " << sch->getStationName() <<
	 "</title><meta name=\"Author\" content=\"KJabata\">" <<
	 "<link rel=\"stylesheet\" href=\"wjabata.css\" type=\"text/css\">" << 
	 "</head>" << endl << "<body " << bodyArgs  << ">" << endl;
      }

      if(withPHPCode)
        cout << 
	"<?  if(file_exists(\"programtop.php\")) require \"programtop.php\" ?>"
	<< endl;

      emisiones[0]=reemisiones[0]=0;
      indE = indR = 0;
      for(int d=1; d<8; d++) {
        for(int h=0; h<24; h++) {
          if((int)i==sch->getBlockPosition(d, h, 0)){
            strip = sch->getCurrentStrip();
            if((!strip->recordingStrip) && (blk->getProgramFile())) {
              str = (char *)&reemisiones;
              ind = indR;
            }
            else {
              str = (char *)&emisiones;
              ind = indE;
            }
						
	    if(str[0]) {
              sprintf(str,"%s <br></b> y los <b>%s DE %d:%02d A %d:%02d", 
	                 str, toDayString(d),
                         strip->beginAtHour,strip->beginAtMinute,
                         strip->endAtHour,strip->endAtMinute);
  	    }
	    else {
              sprintf(str,"%s DE %d:%02d A %d:%02d", toDayString(d),
                         strip->beginAtHour,strip->beginAtMinute,
                         strip->endAtHour,strip->endAtMinute);
            }
          }
          diff = sch->getCurrentStrip()->endAtHour - sch->getCurrentStrip()->beginAtHour;
          if(diff>1) h+=(diff - 1);
        }
      }

      cout << "<table border=1 cellspacing=4 cellpadding=8>" << endl;
      cout << "<tr>" << endl;
      if(blk->getBackImage()!=NULL)
        cout << " <td background=\"" << blk->getBackImage() << "\">" << endl;
      else
        cout << " <td background=\"img/fclaro.png\">" << endl;

      cout << " <font color=#551500 size=2><b>";
      if(emisiones[0])
        cout << emisiones;
      else {
        if(reemisiones[0]) {
          cout << reemisiones;
	  reemisiones[0]=0;
	}
	else
          cout << "Hist&oacute;rico" << endl;
      }

	
      cout << "</b></font><HR>" << endl;

      cout << " <center><font color=#551500 size=5>" << blk->getName() << "</font></center><HR>" << endl;
      if(blk->getImage()!=NULL)
        cout << " <img src=\"" << blk->getImage() << "\" hspace=10 align=\"left\" alt=\"Imagen " << blk->getName() << "\">" << endl;

      cout << " <p><font color=#000000><b>" << blk->getDescription() << "</b></font></p>" << endl;
      cout << " <br clear=\"left\"><br>";

      if(blk->getJabatos()!=NULL)
        cout << "<p>Culpable/s: <b>" << blk->getJabatos() << "</b></p>" << endl;

      if(blk->getEmail()!=NULL)
        cout << "<p>Correo electr&oacute;nico: <a href=\"mailto:" << blk->getEmail() << "\">"
             << blk->getEmail() << "</a></p>" << endl;

      if(blk->getHRef()!=NULL)
        cout << "<p>P&aacute;gina G&uuml;eb: <a href=\"" << blk->getHRef() << "\">"
             << blk->getHRef() << "</a></p>" << endl;

      if((blk->getProgramFile()) ||(reemisiones[0])) {
        cout << "<HR><font color=#551500 size=2><div align=\"right\"><b>" << blk->getName() << "</b>";
        if(reemisiones[0])
          cout << " se repite los <b>" << reemisiones;
        else
          cout << " sonar&aacute; como parte de <b>"
               << sch->getDefaultBlock()->getName();
      }
      cout << "</b></div></font>" << endl;

      if((withPHPCode) && (blk->getProgramFile())) {
        cout << "<hr><font color=#000055 size=2>" << endl;
        sound.setFullPath(blk->getProgramFile());
        cout << "<? echoDownloads(\"" << sound.getName() << ".mp3\"); ?>" << endl;
        cout << "</font>" << endl;

      }

      cout << "</td></table>" << endl;
      break;
    }
  }

  if(withPHPCode)
    cout << "<?  if(file_exists(\"programbottom.php\")) require \"programbottom.php\" ?>" << endl;

  if(!withoutBody) {
    cout << "</body>" << endl;
    cout << "</html>" << endl;
  }
}


void
ParrillaJabata::programsHTMLLinks(char *file) {
  JabatoBlock *blk;

  for(uint i=1; i<sch->blocksCount(); i++) {
    blk = sch->getBlockAt(i);

    if((blk->getId()>0)&&(blk->getId()<1000))   {
      if(allInOne) {
        if(withPHPCode)
          cout << "<a href=\"programas.php#" << i << "\">";
 		    else
          cout << "<a href=\"programas.html#" << i << "\">";
      }
      else {
        if(withPHPCode)
					cout << "<a href=\"" << blk->getId() << ".php\">";
    		else
		      cout << "<a href=\"" << blk->getId() << ".html\">";
      }
			if(linkscolor)
			  cout << "<font color=\"" << linkscolor << "\" >";
			if(i%2)
			  cout << "<b>" << blk->getName() << "</b>";
			else
			  cout << "<i>" << blk->getName() << "</i>";
			if(linkscolor)
			  cout << "</font>";
			cout << "</a>&nbsp;&nbsp;" << endl;
    }
  }
}


void
ParrillaJabata::programRawIDs(char *file) {
  JabatoBlock *blk;

  for(uint i=1; i<sch->blocksCount(); i++) {
    blk = sch->getBlockAt(i);

    if((blk->getId()>0)&&(blk->getId()<1000))   {
			cout << blk->getId() << " ";
    }
  }
}

